# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_SEPARATE_METADATA = Metadata(
    id="2d80a0fa0d07f262c86f7f51153d3da319275b60.workbench",
    name="cifti-separate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiSeparateRoiParamsDictNoTag = typing.TypedDict('_CiftiSeparateRoiParamsDictNoTag', {
    "roi-out": str,
})
CiftiSeparateRoiParamsDictTagged = typing.TypedDict('CiftiSeparateRoiParamsDictTagged', {
    "@type": typing.Literal["roi"],
    "roi-out": str,
})
CiftiSeparateRoiParamsDict = _CiftiSeparateRoiParamsDictNoTag | CiftiSeparateRoiParamsDictTagged


_CiftiSeparateLabelParamsDictNoTag = typing.TypedDict('_CiftiSeparateLabelParamsDictNoTag', {
    "label-out": str,
})
CiftiSeparateLabelParamsDictTagged = typing.TypedDict('CiftiSeparateLabelParamsDictTagged', {
    "@type": typing.Literal["label"],
    "label-out": str,
})
CiftiSeparateLabelParamsDict = _CiftiSeparateLabelParamsDictNoTag | CiftiSeparateLabelParamsDictTagged


_CiftiSeparateVolumeAllParamsDictNoTag = typing.TypedDict('_CiftiSeparateVolumeAllParamsDictNoTag', {
    "volume-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict | None],
    "label": typing.NotRequired[CiftiSeparateLabelParamsDict | None],
    "crop": bool,
})
CiftiSeparateVolumeAllParamsDictTagged = typing.TypedDict('CiftiSeparateVolumeAllParamsDictTagged', {
    "@type": typing.Literal["volume-all"],
    "volume-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict | None],
    "label": typing.NotRequired[CiftiSeparateLabelParamsDict | None],
    "crop": bool,
})
CiftiSeparateVolumeAllParamsDict = _CiftiSeparateVolumeAllParamsDictNoTag | CiftiSeparateVolumeAllParamsDictTagged


_CiftiSeparateRoiParamsDict_NoTag = typing.TypedDict('_CiftiSeparateRoiParamsDict_NoTag', {
    "roi-out": str,
})
CiftiSeparateRoiParamsDictTagged_ = typing.TypedDict('CiftiSeparateRoiParamsDictTagged_', {
    "@type": typing.Literal["roi"],
    "roi-out": str,
})
CiftiSeparateRoiParamsDict_ = _CiftiSeparateRoiParamsDict_NoTag | CiftiSeparateRoiParamsDictTagged_


_CiftiSeparateLabelParamsDict_NoTag = typing.TypedDict('_CiftiSeparateLabelParamsDict_NoTag', {
    "structure": str,
    "label-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_ | None],
})
CiftiSeparateLabelParamsDictTagged_ = typing.TypedDict('CiftiSeparateLabelParamsDictTagged_', {
    "@type": typing.Literal["label"],
    "structure": str,
    "label-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_ | None],
})
CiftiSeparateLabelParamsDict_ = _CiftiSeparateLabelParamsDict_NoTag | CiftiSeparateLabelParamsDictTagged_


_CiftiSeparateRoiParamsDict_2NoTag = typing.TypedDict('_CiftiSeparateRoiParamsDict_2NoTag', {
    "roi-out": str,
})
CiftiSeparateRoiParamsDictTagged_2 = typing.TypedDict('CiftiSeparateRoiParamsDictTagged_2', {
    "@type": typing.Literal["roi"],
    "roi-out": str,
})
CiftiSeparateRoiParamsDict_2 = _CiftiSeparateRoiParamsDict_2NoTag | CiftiSeparateRoiParamsDictTagged_2


_CiftiSeparateMetricParamsDictNoTag = typing.TypedDict('_CiftiSeparateMetricParamsDictNoTag', {
    "structure": str,
    "metric-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_2 | None],
})
CiftiSeparateMetricParamsDictTagged = typing.TypedDict('CiftiSeparateMetricParamsDictTagged', {
    "@type": typing.Literal["metric"],
    "structure": str,
    "metric-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_2 | None],
})
CiftiSeparateMetricParamsDict = _CiftiSeparateMetricParamsDictNoTag | CiftiSeparateMetricParamsDictTagged


_CiftiSeparateRoiParamsDict_3NoTag = typing.TypedDict('_CiftiSeparateRoiParamsDict_3NoTag', {
    "roi-out": str,
})
CiftiSeparateRoiParamsDictTagged_3 = typing.TypedDict('CiftiSeparateRoiParamsDictTagged_3', {
    "@type": typing.Literal["roi"],
    "roi-out": str,
})
CiftiSeparateRoiParamsDict_3 = _CiftiSeparateRoiParamsDict_3NoTag | CiftiSeparateRoiParamsDictTagged_3


_CiftiSeparateVolumeParamsDictNoTag = typing.TypedDict('_CiftiSeparateVolumeParamsDictNoTag', {
    "structure": str,
    "volume-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_3 | None],
    "crop": bool,
})
CiftiSeparateVolumeParamsDictTagged = typing.TypedDict('CiftiSeparateVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "structure": str,
    "volume-out": str,
    "roi": typing.NotRequired[CiftiSeparateRoiParamsDict_3 | None],
    "crop": bool,
})
CiftiSeparateVolumeParamsDict = _CiftiSeparateVolumeParamsDictNoTag | CiftiSeparateVolumeParamsDictTagged


_CiftiSeparateParamsDictNoTag = typing.TypedDict('_CiftiSeparateParamsDictNoTag', {
    "cifti-in": InputPathType,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiSeparateVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiSeparateLabelParamsDict_] | None],
    "metric": typing.NotRequired[list[CiftiSeparateMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiSeparateVolumeParamsDict] | None],
})
CiftiSeparateParamsDictTagged = typing.TypedDict('CiftiSeparateParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-separate"],
    "cifti-in": InputPathType,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiSeparateVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiSeparateLabelParamsDict_] | None],
    "metric": typing.NotRequired[list[CiftiSeparateMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiSeparateVolumeParamsDict] | None],
})
CiftiSeparateParamsDict = _CiftiSeparateParamsDictNoTag | CiftiSeparateParamsDictTagged


class CiftiSeparateRoiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateRoiParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_out: OutputPathType
    """the roi output volume"""


def cifti_separate_roi(
    roi_out: str,
) -> CiftiSeparateRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_out: the roi output volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-out": roi_out,
    }
    return params


def cifti_separate_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-out", None) is None:
        raise StyxValidationError("`roi-out` must not be None")
    if not isinstance(params["roi-out"], str):
        raise StyxValidationError(f'`roi-out` has the wrong type: Received `{type(params.get("roi-out", None))}` expected `str`')


def cifti_separate_roi_cargs(
    params: CiftiSeparateRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        params.get("roi-out", None)
    ])
    return cargs


def cifti_separate_roi_outputs(
    params: CiftiSeparateRoiParamsDict,
    execution: Execution,
) -> CiftiSeparateRoiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateRoiOutputs(
        root=execution.output_file("."),
        roi_out=execution.output_file(params.get("roi-out", None)),
    )
    return ret


class CiftiSeparateLabelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateLabelParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the label output volume"""


def cifti_separate_label(
    label_out: str,
) -> CiftiSeparateLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_out: the label output volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "label-out": label_out,
    }
    return params


def cifti_separate_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')


def cifti_separate_label_cargs(
    params: CiftiSeparateLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        params.get("label-out", None)
    ])
    return cargs


def cifti_separate_label_outputs(
    params: CiftiSeparateLabelParamsDict,
    execution: Execution,
) -> CiftiSeparateLabelOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateLabelOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


class CiftiSeparateVolumeAllOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateVolumeAllParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""
    roi: CiftiSeparateRoiOutputs | None
    """Outputs from `cifti_separate_roi_outputs`."""
    label: CiftiSeparateLabelOutputs | None
    """Outputs from `cifti_separate_label_outputs`."""


def cifti_separate_volume_all(
    volume_out: str,
    roi: CiftiSeparateRoiParamsDict | None = None,
    label: CiftiSeparateLabelParamsDict | None = None,
    crop: bool = False,
) -> CiftiSeparateVolumeAllParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_out: the output volume.
        roi: also output the roi of which voxels have data.
        label: output a volume label file indicating the location of structures.
        crop: crop volume to the size of the data rather than using the\
            original volume size.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-all",
        "volume-out": volume_out,
        "crop": crop,
    }
    if roi is not None:
        params["roi"] = roi
    if label is not None:
        params["label"] = label
    return params


def cifti_separate_volume_all_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateVolumeAllParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("roi", None) is not None:
        cifti_separate_roi_validate(params["roi"])
    if params.get("label", None) is not None:
        cifti_separate_label_validate(params["label"])
    if params.get("crop", False) is None:
        raise StyxValidationError("`crop` must not be None")
    if not isinstance(params["crop"], bool):
        raise StyxValidationError(f'`crop` has the wrong type: Received `{type(params.get("crop", False))}` expected `bool`')


def cifti_separate_volume_all_cargs(
    params: CiftiSeparateVolumeAllParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-all",
        params.get("volume-out", None),
        *(cifti_separate_roi_cargs(params.get("roi", None), execution) if (params.get("roi", None) is not None) else []),
        *(cifti_separate_label_cargs(params.get("label", None), execution) if (params.get("label", None) is not None) else [])
    ])
    if params.get("crop", False):
        cargs.append("-crop")
    return cargs


def cifti_separate_volume_all_outputs(
    params: CiftiSeparateVolumeAllParamsDict,
    execution: Execution,
) -> CiftiSeparateVolumeAllOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateVolumeAllOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
        roi=cifti_separate_roi_outputs(params.get("roi"), execution) if params.get("roi") else None,
        label=cifti_separate_label_outputs(params.get("label"), execution) if params.get("label") else None,
    )
    return ret


class CiftiSeparateRoiOutputs_(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateRoiParamsDict_ | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_out: OutputPathType
    """the roi output metric"""


def cifti_separate_roi_(
    roi_out: str,
) -> CiftiSeparateRoiParamsDictTagged_:
    """
    Build parameters.
    
    Args:
        roi_out: the roi output metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-out": roi_out,
    }
    return params


def cifti_separate_roi_validate_(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateRoiParamsDict_` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-out", None) is None:
        raise StyxValidationError("`roi-out` must not be None")
    if not isinstance(params["roi-out"], str):
        raise StyxValidationError(f'`roi-out` has the wrong type: Received `{type(params.get("roi-out", None))}` expected `str`')


def cifti_separate_roi_cargs_(
    params: CiftiSeparateRoiParamsDict_,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        params.get("roi-out", None)
    ])
    return cargs


def cifti_separate_roi_outputs_(
    params: CiftiSeparateRoiParamsDict_,
    execution: Execution,
) -> CiftiSeparateRoiOutputs_:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateRoiOutputs_(
        root=execution.output_file("."),
        roi_out=execution.output_file(params.get("roi-out", None)),
    )
    return ret


class CiftiSeparateLabelOutputs_(typing.NamedTuple):
    """
    Output object returned when calling `list[CiftiSeparateLabelParamsDict_] | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label file"""
    roi: CiftiSeparateRoiOutputs_ | None
    """Outputs from `cifti_separate_roi_outputs_`."""


def cifti_separate_label_(
    structure: str,
    label_out: str,
    roi: CiftiSeparateRoiParamsDict_ | None = None,
) -> CiftiSeparateLabelParamsDictTagged_:
    """
    Build parameters.
    
    Args:
        structure: the structure to output.
        label_out: the output label file.
        roi: also output the roi of which vertices have data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "structure": structure,
        "label-out": label_out,
    }
    if roi is not None:
        params["roi"] = roi
    return params


def cifti_separate_label_validate_(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateLabelParamsDict_` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("roi", None) is not None:
        cifti_separate_roi_validate_(params["roi"])


def cifti_separate_label_cargs_(
    params: CiftiSeparateLabelParamsDict_,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        params.get("structure", None),
        params.get("label-out", None),
        *(cifti_separate_roi_cargs_(params.get("roi", None), execution) if (params.get("roi", None) is not None) else [])
    ])
    return cargs


def cifti_separate_label_outputs_(
    params: CiftiSeparateLabelParamsDict_,
    execution: Execution,
) -> CiftiSeparateLabelOutputs_:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateLabelOutputs_(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
        roi=cifti_separate_roi_outputs_(params.get("roi"), execution) if params.get("roi") else None,
    )
    return ret


class CiftiSeparateRoiOutputs_2(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateRoiParamsDict_2 | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_out: OutputPathType
    """the roi output metric"""


def cifti_separate_roi_2(
    roi_out: str,
) -> CiftiSeparateRoiParamsDictTagged_2:
    """
    Build parameters.
    
    Args:
        roi_out: the roi output metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-out": roi_out,
    }
    return params


def cifti_separate_roi_validate_2(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateRoiParamsDict_2` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-out", None) is None:
        raise StyxValidationError("`roi-out` must not be None")
    if not isinstance(params["roi-out"], str):
        raise StyxValidationError(f'`roi-out` has the wrong type: Received `{type(params.get("roi-out", None))}` expected `str`')


def cifti_separate_roi_cargs_2(
    params: CiftiSeparateRoiParamsDict_2,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        params.get("roi-out", None)
    ])
    return cargs


def cifti_separate_roi_outputs_2(
    params: CiftiSeparateRoiParamsDict_2,
    execution: Execution,
) -> CiftiSeparateRoiOutputs_2:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateRoiOutputs_2(
        root=execution.output_file("."),
        roi_out=execution.output_file(params.get("roi-out", None)),
    )
    return ret


class CiftiSeparateMetricOutputs(typing.NamedTuple):
    """
    Output object returned when calling `list[CiftiSeparateMetricParamsDict] | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""
    roi: CiftiSeparateRoiOutputs_2 | None
    """Outputs from `cifti_separate_roi_outputs_2`."""


def cifti_separate_metric(
    structure: str,
    metric_out: str,
    roi: CiftiSeparateRoiParamsDict_2 | None = None,
) -> CiftiSeparateMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to output.
        metric_out: the output metric.
        roi: also output the roi of which vertices have data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "metric",
        "structure": structure,
        "metric-out": metric_out,
    }
    if roi is not None:
        params["roi"] = roi
    return params


def cifti_separate_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("roi", None) is not None:
        cifti_separate_roi_validate_2(params["roi"])


def cifti_separate_metric_cargs(
    params: CiftiSeparateMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-metric",
        params.get("structure", None),
        params.get("metric-out", None),
        *(cifti_separate_roi_cargs_2(params.get("roi", None), execution) if (params.get("roi", None) is not None) else [])
    ])
    return cargs


def cifti_separate_metric_outputs(
    params: CiftiSeparateMetricParamsDict,
    execution: Execution,
) -> CiftiSeparateMetricOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateMetricOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
        roi=cifti_separate_roi_outputs_2(params.get("roi"), execution) if params.get("roi") else None,
    )
    return ret


class CiftiSeparateRoiOutputs_3(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateRoiParamsDict_3 | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_out: OutputPathType
    """the roi output volume"""


def cifti_separate_roi_3(
    roi_out: str,
) -> CiftiSeparateRoiParamsDictTagged_3:
    """
    Build parameters.
    
    Args:
        roi_out: the roi output volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-out": roi_out,
    }
    return params


def cifti_separate_roi_validate_3(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateRoiParamsDict_3` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-out", None) is None:
        raise StyxValidationError("`roi-out` must not be None")
    if not isinstance(params["roi-out"], str):
        raise StyxValidationError(f'`roi-out` has the wrong type: Received `{type(params.get("roi-out", None))}` expected `str`')


def cifti_separate_roi_cargs_3(
    params: CiftiSeparateRoiParamsDict_3,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        params.get("roi-out", None)
    ])
    return cargs


def cifti_separate_roi_outputs_3(
    params: CiftiSeparateRoiParamsDict_3,
    execution: Execution,
) -> CiftiSeparateRoiOutputs_3:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateRoiOutputs_3(
        root=execution.output_file("."),
        roi_out=execution.output_file(params.get("roi-out", None)),
    )
    return ret


class CiftiSeparateVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `list[CiftiSeparateVolumeParamsDict] | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""
    roi: CiftiSeparateRoiOutputs_3 | None
    """Outputs from `cifti_separate_roi_outputs_3`."""


def cifti_separate_volume(
    structure: str,
    volume_out: str,
    roi: CiftiSeparateRoiParamsDict_3 | None = None,
    crop: bool = False,
) -> CiftiSeparateVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to output.
        volume_out: the output volume.
        roi: also output the roi of which voxels have data.
        crop: crop volume to the size of the component rather than using the\
            original volume size.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "structure": structure,
        "volume-out": volume_out,
        "crop": crop,
    }
    if roi is not None:
        params["roi"] = roi
    return params


def cifti_separate_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("roi", None) is not None:
        cifti_separate_roi_validate_3(params["roi"])
    if params.get("crop", False) is None:
        raise StyxValidationError("`crop` must not be None")
    if not isinstance(params["crop"], bool):
        raise StyxValidationError(f'`crop` has the wrong type: Received `{type(params.get("crop", False))}` expected `bool`')


def cifti_separate_volume_cargs(
    params: CiftiSeparateVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        params.get("structure", None),
        params.get("volume-out", None),
        *(cifti_separate_roi_cargs_3(params.get("roi", None), execution) if (params.get("roi", None) is not None) else [])
    ])
    if params.get("crop", False):
        cargs.append("-crop")
    return cargs


def cifti_separate_volume_outputs(
    params: CiftiSeparateVolumeParamsDict,
    execution: Execution,
) -> CiftiSeparateVolumeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateVolumeOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
        roi=cifti_separate_roi_outputs_3(params.get("roi"), execution) if params.get("roi") else None,
    )
    return ret


class CiftiSeparateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiSeparateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_all: CiftiSeparateVolumeAllOutputs | None
    """Outputs from `cifti_separate_volume_all_outputs`."""
    label: list[CiftiSeparateLabelOutputs_] | None
    """Outputs from `cifti_separate_label_outputs_`.This is a list of outputs
    with the same length and order as the inputs."""
    metric: list[CiftiSeparateMetricOutputs] | None
    """Outputs from `cifti_separate_metric_outputs`.This is a list of outputs
    with the same length and order as the inputs."""
    volume: list[CiftiSeparateVolumeOutputs] | None
    """Outputs from `cifti_separate_volume_outputs`.This is a list of outputs
    with the same length and order as the inputs."""


def cifti_separate_params(
    cifti_in: InputPathType,
    direction: str,
    volume_all: CiftiSeparateVolumeAllParamsDict | None = None,
    label: list[CiftiSeparateLabelParamsDict_] | None = None,
    metric: list[CiftiSeparateMetricParamsDict] | None = None,
    volume: list[CiftiSeparateVolumeParamsDict] | None = None,
) -> CiftiSeparateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti to separate a component of.
        direction: which direction to separate into components, ROW or COLUMN.
        volume_all: separate all volume structures into a volume file.
        label: separate a surface model into a surface label file.
        metric: separate a surface model into a metric file.
        volume: separate a volume structure into a volume file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-separate",
        "cifti-in": cifti_in,
        "direction": direction,
    }
    if volume_all is not None:
        params["volume-all"] = volume_all
    if label is not None:
        params["label"] = label
    if metric is not None:
        params["metric"] = metric
    if volume is not None:
        params["volume"] = volume
    return params


def cifti_separate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiSeparateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("volume-all", None) is not None:
        cifti_separate_volume_all_validate(params["volume-all"])
    if params.get("label", None) is not None:
        if not isinstance(params["label"], list):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `list[CiftiSeparateLabelParamsDict_] | None`')
        for e in params["label"]:
            cifti_separate_label_validate_(e)
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], list):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `list[CiftiSeparateMetricParamsDict] | None`')
        for e in params["metric"]:
            cifti_separate_metric_validate(e)
    if params.get("volume", None) is not None:
        if not isinstance(params["volume"], list):
            raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `list[CiftiSeparateVolumeParamsDict] | None`')
        for e in params["volume"]:
            cifti_separate_volume_validate(e)


def cifti_separate_cargs(
    params: CiftiSeparateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-separate"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("direction", None))
    if params.get("volume-all", None) is not None or params.get("label", None) is not None or params.get("metric", None) is not None or params.get("volume", None) is not None:
        cargs.extend([
            *(cifti_separate_volume_all_cargs(params.get("volume-all", None), execution) if (params.get("volume-all", None) is not None) else []),
            *([a for c in [cifti_separate_label_cargs_(s, execution) for s in params.get("label", None)] for a in c] if (params.get("label", None) is not None) else []),
            *([a for c in [cifti_separate_metric_cargs(s, execution) for s in params.get("metric", None)] for a in c] if (params.get("metric", None) is not None) else []),
            *([a for c in [cifti_separate_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c] if (params.get("volume", None) is not None) else [])
        ])
    return cargs


def cifti_separate_outputs(
    params: CiftiSeparateParamsDict,
    execution: Execution,
) -> CiftiSeparateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiSeparateOutputs(
        root=execution.output_file("."),
        volume_all=cifti_separate_volume_all_outputs(params.get("volume-all"), execution) if params.get("volume-all") else None,
        label=[cifti_separate_label_outputs_(i, execution) if cifti_separate_label_outputs_ else None for i in params.get("label")] if params.get("label") else None,
        metric=[cifti_separate_metric_outputs(i, execution) if cifti_separate_metric_outputs else None for i in params.get("metric")] if params.get("metric") else None,
        volume=[cifti_separate_volume_outputs(i, execution) if cifti_separate_volume_outputs else None for i in params.get("volume")] if params.get("volume") else None,
    )
    return ret


def cifti_separate_execute(
    params: CiftiSeparateParamsDict,
    runner: Runner | None = None,
) -> CiftiSeparateOutputs:
    """
    WRITE A CIFTI STRUCTURE AS METRIC, LABEL OR VOLUME.
    
    For dtseries, dscalar, and dlabel, use COLUMN for <direction>, and if you
    have a symmetric dconn, COLUMN is more efficient.
    
    You must specify at least one of -metric, -volume-all, -volume, or -label
    for this command to do anything. Output volumes will spatially line up with
    their original positions, whether or not they are cropped. Volume files
    produced by separating a dlabel file, or from the -label suboption of
    -volume-all, will be label volumes, see -volume-help.
    
    For each <structure> argument, use one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiSeparateOutputs`).
    """
    cifti_separate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_SEPARATE_METADATA)
    params = execution.params(params)
    cargs = cifti_separate_cargs(params, execution)
    ret = cifti_separate_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_separate(
    cifti_in: InputPathType,
    direction: str,
    volume_all: CiftiSeparateVolumeAllParamsDict | None = None,
    label: list[CiftiSeparateLabelParamsDict_] | None = None,
    metric: list[CiftiSeparateMetricParamsDict] | None = None,
    volume: list[CiftiSeparateVolumeParamsDict] | None = None,
    runner: Runner | None = None,
) -> CiftiSeparateOutputs:
    """
    WRITE A CIFTI STRUCTURE AS METRIC, LABEL OR VOLUME.
    
    For dtseries, dscalar, and dlabel, use COLUMN for <direction>, and if you
    have a symmetric dconn, COLUMN is more efficient.
    
    You must specify at least one of -metric, -volume-all, -volume, or -label
    for this command to do anything. Output volumes will spatially line up with
    their original positions, whether or not they are cropped. Volume files
    produced by separating a dlabel file, or from the -label suboption of
    -volume-all, will be label volumes, see -volume-help.
    
    For each <structure> argument, use one of the following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti_in: the cifti to separate a component of.
        direction: which direction to separate into components, ROW or COLUMN.
        volume_all: separate all volume structures into a volume file.
        label: separate a surface model into a surface label file.
        metric: separate a surface model into a metric file.
        volume: separate a volume structure into a volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiSeparateOutputs`).
    """
    params = cifti_separate_params(
        cifti_in=cifti_in,
        direction=direction,
        volume_all=volume_all,
        label=label,
        metric=metric,
        volume=volume,
    )
    return cifti_separate_execute(params, runner)


__all__ = [
    "CIFTI_SEPARATE_METADATA",
    "CiftiSeparateLabelOutputs",
    "CiftiSeparateLabelOutputs_",
    "CiftiSeparateLabelParamsDict",
    "CiftiSeparateLabelParamsDictTagged",
    "CiftiSeparateLabelParamsDictTagged_",
    "CiftiSeparateLabelParamsDict_",
    "CiftiSeparateMetricOutputs",
    "CiftiSeparateMetricParamsDict",
    "CiftiSeparateMetricParamsDictTagged",
    "CiftiSeparateOutputs",
    "CiftiSeparateParamsDict",
    "CiftiSeparateParamsDictTagged",
    "CiftiSeparateRoiOutputs",
    "CiftiSeparateRoiOutputs_",
    "CiftiSeparateRoiOutputs_2",
    "CiftiSeparateRoiOutputs_3",
    "CiftiSeparateRoiParamsDict",
    "CiftiSeparateRoiParamsDictTagged",
    "CiftiSeparateRoiParamsDictTagged_",
    "CiftiSeparateRoiParamsDictTagged_2",
    "CiftiSeparateRoiParamsDictTagged_3",
    "CiftiSeparateRoiParamsDict_",
    "CiftiSeparateRoiParamsDict_2",
    "CiftiSeparateRoiParamsDict_3",
    "CiftiSeparateVolumeAllOutputs",
    "CiftiSeparateVolumeAllParamsDict",
    "CiftiSeparateVolumeAllParamsDictTagged",
    "CiftiSeparateVolumeOutputs",
    "CiftiSeparateVolumeParamsDict",
    "CiftiSeparateVolumeParamsDictTagged",
    "cifti_separate",
    "cifti_separate_execute",
    "cifti_separate_label",
    "cifti_separate_label_",
    "cifti_separate_metric",
    "cifti_separate_params",
    "cifti_separate_roi",
    "cifti_separate_roi_",
    "cifti_separate_roi_2",
    "cifti_separate_roi_3",
    "cifti_separate_volume",
    "cifti_separate_volume_all",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_TO_BORDER_METADATA = Metadata(
    id="73238edbf5a2ddc85d7db5f72cf5459fa04f8d95.workbench",
    name="label-to-border",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelToBorderParamsDictNoTag = typing.TypedDict('_LabelToBorderParamsDictNoTag', {
    "surface": InputPathType,
    "label-in": InputPathType,
    "border-out": str,
    "column": typing.NotRequired[str | None],
    "fraction": typing.NotRequired[float | None],
})
LabelToBorderParamsDictTagged = typing.TypedDict('LabelToBorderParamsDictTagged', {
    "@type": typing.Literal["workbench/label-to-border"],
    "surface": InputPathType,
    "label-in": InputPathType,
    "border-out": str,
    "column": typing.NotRequired[str | None],
    "fraction": typing.NotRequired[float | None],
})
LabelToBorderParamsDict = _LabelToBorderParamsDictNoTag | LabelToBorderParamsDictTagged


class LabelToBorderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelToBorderParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    border_out: OutputPathType
    """the output border file"""


def label_to_border_params(
    surface: InputPathType,
    label_in: InputPathType,
    border_out: str,
    column: str | None = None,
    fraction: float | None = None,
) -> LabelToBorderParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for neighbor information.
        label_in: the input label file.
        border_out: the output border file.
        column: select a single column\
            \
            the column number or name.
        fraction: set how far along the edge border points are drawn\
            \
            fraction along edge from inside vertex (default 0.33).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-to-border",
        "surface": surface,
        "label-in": label_in,
        "border-out": border_out,
    }
    if column is not None:
        params["column"] = column
    if fraction is not None:
        params["fraction"] = fraction
    return params


def label_to_border_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelToBorderParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("border-out", None) is None:
        raise StyxValidationError("`border-out` must not be None")
    if not isinstance(params["border-out"], str):
        raise StyxValidationError(f'`border-out` has the wrong type: Received `{type(params.get("border-out", None))}` expected `str`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("fraction", None) is not None:
        if not isinstance(params["fraction"], (float, int)):
            raise StyxValidationError(f'`fraction` has the wrong type: Received `{type(params.get("fraction", None))}` expected `float | None`')


def label_to_border_cargs(
    params: LabelToBorderParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-to-border"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("border-out", None))
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("fraction", None) is not None:
        cargs.extend([
            "-placement",
            str(params.get("fraction", None))
        ])
    return cargs


def label_to_border_outputs(
    params: LabelToBorderParamsDict,
    execution: Execution,
) -> LabelToBorderOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelToBorderOutputs(
        root=execution.output_file("."),
        border_out=execution.output_file(params.get("border-out", None)),
    )
    return ret


def label_to_border_execute(
    params: LabelToBorderParamsDict,
    runner: Runner | None = None,
) -> LabelToBorderOutputs:
    """
    DRAW BORDERS AROUND LABELS.
    
    For each label, finds all edges on the mesh that cross the boundary of the
    label, and draws borders through them. By default, this is done on all
    columns in the input file, using the map name as the class name for the
    border.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelToBorderOutputs`).
    """
    label_to_border_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_TO_BORDER_METADATA)
    params = execution.params(params)
    cargs = label_to_border_cargs(params, execution)
    ret = label_to_border_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_to_border(
    surface: InputPathType,
    label_in: InputPathType,
    border_out: str,
    column: str | None = None,
    fraction: float | None = None,
    runner: Runner | None = None,
) -> LabelToBorderOutputs:
    """
    DRAW BORDERS AROUND LABELS.
    
    For each label, finds all edges on the mesh that cross the boundary of the
    label, and draws borders through them. By default, this is done on all
    columns in the input file, using the map name as the class name for the
    border.
    
    Args:
        surface: the surface to use for neighbor information.
        label_in: the input label file.
        border_out: the output border file.
        column: select a single column\
            \
            the column number or name.
        fraction: set how far along the edge border points are drawn\
            \
            fraction along edge from inside vertex (default 0.33).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelToBorderOutputs`).
    """
    params = label_to_border_params(
        surface=surface,
        label_in=label_in,
        border_out=border_out,
        column=column,
        fraction=fraction,
    )
    return label_to_border_execute(params, runner)


__all__ = [
    "LABEL_TO_BORDER_METADATA",
    "LabelToBorderOutputs",
    "LabelToBorderParamsDict",
    "LabelToBorderParamsDictTagged",
    "label_to_border",
    "label_to_border_execute",
    "label_to_border_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SET_STRUCTURE_METADATA = Metadata(
    id="3f017d3648fe7b335b883d3f5dcd0884cd807980.workbench",
    name="set-structure",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SetStructureParamsDictNoTag = typing.TypedDict('_SetStructureParamsDictNoTag', {
    "data-file": str,
    "structure": str,
    "secondary type": typing.NotRequired[str | None],
    "type": typing.NotRequired[str | None],
})
SetStructureParamsDictTagged = typing.TypedDict('SetStructureParamsDictTagged', {
    "@type": typing.Literal["workbench/set-structure"],
    "data-file": str,
    "structure": str,
    "secondary type": typing.NotRequired[str | None],
    "type": typing.NotRequired[str | None],
})
SetStructureParamsDict = _SetStructureParamsDictNoTag | SetStructureParamsDictTagged


class SetStructureOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SetStructureParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def set_structure_params(
    data_file: str,
    structure: str,
    secondary_type: str | None = None,
    type_: str | None = None,
) -> SetStructureParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_file: the file to set the structure of.
        structure: the structure to set the file to.
        secondary_type: set the secondary type of a surface (only used if file\
            is a surface file)\
            \
            name of surface secondary type.
        type_: set the type of a surface (only used if file is a surface file)\
            \
            name of surface type.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/set-structure",
        "data-file": data_file,
        "structure": structure,
    }
    if secondary_type is not None:
        params["secondary type"] = secondary_type
    if type_ is not None:
        params["type"] = type_
    return params


def set_structure_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SetStructureParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-file", None) is None:
        raise StyxValidationError("`data-file` must not be None")
    if not isinstance(params["data-file"], str):
        raise StyxValidationError(f'`data-file` has the wrong type: Received `{type(params.get("data-file", None))}` expected `str`')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("secondary type", None) is not None:
        if not isinstance(params["secondary type"], str):
            raise StyxValidationError(f'`secondary type` has the wrong type: Received `{type(params.get("secondary type", None))}` expected `str | None`')
    if params.get("type", None) is not None:
        if not isinstance(params["type"], str):
            raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str | None`')


def set_structure_cargs(
    params: SetStructureParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-set-structure"
    ])
    cargs.append(params.get("data-file", None))
    cargs.append(params.get("structure", None))
    if params.get("secondary type", None) is not None:
        cargs.extend([
            "-surface-secondary-type",
            params.get("secondary type", None)
        ])
    if params.get("type", None) is not None:
        cargs.extend([
            "-surface-type",
            params.get("type", None)
        ])
    return cargs


def set_structure_outputs(
    params: SetStructureParamsDict,
    execution: Execution,
) -> SetStructureOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SetStructureOutputs(
        root=execution.output_file("."),
    )
    return ret


def set_structure_execute(
    params: SetStructureParamsDict,
    runner: Runner | None = None,
) -> SetStructureOutputs:
    """
    SET STRUCTURE OF A DATA FILE.
    
    The existing file is modified and rewritten to the same filename. Valid
    values for the structure name are:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    
    Valid names for the surface type are:
    
    UNKNOWN
    RECONSTRUCTION
    ANATOMICAL
    INFLATED
    VERY_INFLATED
    SPHERICAL
    SEMI_SPHERICAL
    ELLIPSOID
    FLAT
    HULL
    
    Valid names for the surface secondary type are:
    
    INVALID
    GRAY_WHITE
    MIDTHICKNESS
    PIAL
    INNER
    OUTER
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetStructureOutputs`).
    """
    set_structure_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SET_STRUCTURE_METADATA)
    params = execution.params(params)
    cargs = set_structure_cargs(params, execution)
    ret = set_structure_outputs(params, execution)
    execution.run(cargs)
    return ret


def set_structure(
    data_file: str,
    structure: str,
    secondary_type: str | None = None,
    type_: str | None = None,
    runner: Runner | None = None,
) -> SetStructureOutputs:
    """
    SET STRUCTURE OF A DATA FILE.
    
    The existing file is modified and rewritten to the same filename. Valid
    values for the structure name are:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    
    Valid names for the surface type are:
    
    UNKNOWN
    RECONSTRUCTION
    ANATOMICAL
    INFLATED
    VERY_INFLATED
    SPHERICAL
    SEMI_SPHERICAL
    ELLIPSOID
    FLAT
    HULL
    
    Valid names for the surface secondary type are:
    
    INVALID
    GRAY_WHITE
    MIDTHICKNESS
    PIAL
    INNER
    OUTER
    .
    
    Args:
        data_file: the file to set the structure of.
        structure: the structure to set the file to.
        secondary_type: set the secondary type of a surface (only used if file\
            is a surface file)\
            \
            name of surface secondary type.
        type_: set the type of a surface (only used if file is a surface file)\
            \
            name of surface type.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SetStructureOutputs`).
    """
    params = set_structure_params(
        data_file=data_file,
        structure=structure,
        secondary_type=secondary_type,
        type_=type_,
    )
    return set_structure_execute(params, runner)


__all__ = [
    "SET_STRUCTURE_METADATA",
    "SetStructureOutputs",
    "SetStructureParamsDict",
    "SetStructureParamsDictTagged",
    "set_structure",
    "set_structure_execute",
    "set_structure_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_CORTEX_LAYER_METADATA = Metadata(
    id="1f7274373b18590446afbc6e9b2ac5914d86c611.workbench",
    name="surface-cortex-layer",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceCortexLayerPlacementOutParamsDictNoTag = typing.TypedDict('_SurfaceCortexLayerPlacementOutParamsDictNoTag', {
    "placement-metric": str,
})
SurfaceCortexLayerPlacementOutParamsDictTagged = typing.TypedDict('SurfaceCortexLayerPlacementOutParamsDictTagged', {
    "@type": typing.Literal["placement-out"],
    "placement-metric": str,
})
SurfaceCortexLayerPlacementOutParamsDict = _SurfaceCortexLayerPlacementOutParamsDictNoTag | SurfaceCortexLayerPlacementOutParamsDictTagged


_SurfaceCortexLayerParamsDictNoTag = typing.TypedDict('_SurfaceCortexLayerParamsDictNoTag', {
    "white-surface": InputPathType,
    "pial-surface": InputPathType,
    "location": float,
    "out-surface": str,
    "placement-out": typing.NotRequired[SurfaceCortexLayerPlacementOutParamsDict | None],
})
SurfaceCortexLayerParamsDictTagged = typing.TypedDict('SurfaceCortexLayerParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-cortex-layer"],
    "white-surface": InputPathType,
    "pial-surface": InputPathType,
    "location": float,
    "out-surface": str,
    "placement-out": typing.NotRequired[SurfaceCortexLayerPlacementOutParamsDict | None],
})
SurfaceCortexLayerParamsDict = _SurfaceCortexLayerParamsDictNoTag | SurfaceCortexLayerParamsDictTagged


class SurfaceCortexLayerPlacementOutOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCortexLayerPlacementOutParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    placement_metric: OutputPathType
    """output metric"""


def surface_cortex_layer_placement_out(
    placement_metric: str,
) -> SurfaceCortexLayerPlacementOutParamsDictTagged:
    """
    Build parameters.
    
    Args:
        placement_metric: output metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "placement-out",
        "placement-metric": placement_metric,
    }
    return params


def surface_cortex_layer_placement_out_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCortexLayerPlacementOutParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("placement-metric", None) is None:
        raise StyxValidationError("`placement-metric` must not be None")
    if not isinstance(params["placement-metric"], str):
        raise StyxValidationError(f'`placement-metric` has the wrong type: Received `{type(params.get("placement-metric", None))}` expected `str`')


def surface_cortex_layer_placement_out_cargs(
    params: SurfaceCortexLayerPlacementOutParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-placement-out",
        params.get("placement-metric", None)
    ])
    return cargs


def surface_cortex_layer_placement_out_outputs(
    params: SurfaceCortexLayerPlacementOutParamsDict,
    execution: Execution,
) -> SurfaceCortexLayerPlacementOutOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCortexLayerPlacementOutOutputs(
        root=execution.output_file("."),
        placement_metric=execution.output_file(params.get("placement-metric", None)),
    )
    return ret


class SurfaceCortexLayerOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCortexLayerParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surface: OutputPathType
    """the output surface"""
    placement_out: SurfaceCortexLayerPlacementOutOutputs | None
    """Outputs from `surface_cortex_layer_placement_out_outputs`."""


def surface_cortex_layer_params(
    white_surface: InputPathType,
    pial_surface: InputPathType,
    location: float,
    out_surface: str,
    placement_out: SurfaceCortexLayerPlacementOutParamsDict | None = None,
) -> SurfaceCortexLayerParamsDictTagged:
    """
    Build parameters.
    
    Args:
        white_surface: the white matter surface.
        pial_surface: the pial surface.
        location: what volume fraction to place the layer at.
        out_surface: the output surface.
        placement_out: output the placement as a volume fraction from pial to\
            white.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-cortex-layer",
        "white-surface": white_surface,
        "pial-surface": pial_surface,
        "location": location,
        "out-surface": out_surface,
    }
    if placement_out is not None:
        params["placement-out"] = placement_out
    return params


def surface_cortex_layer_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCortexLayerParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("white-surface", None) is None:
        raise StyxValidationError("`white-surface` must not be None")
    if not isinstance(params["white-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`white-surface` has the wrong type: Received `{type(params.get("white-surface", None))}` expected `InputPathType`')
    if params.get("pial-surface", None) is None:
        raise StyxValidationError("`pial-surface` must not be None")
    if not isinstance(params["pial-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`pial-surface` has the wrong type: Received `{type(params.get("pial-surface", None))}` expected `InputPathType`')
    if params.get("location", None) is None:
        raise StyxValidationError("`location` must not be None")
    if not isinstance(params["location"], (float, int)):
        raise StyxValidationError(f'`location` has the wrong type: Received `{type(params.get("location", None))}` expected `float`')
    if params.get("out-surface", None) is None:
        raise StyxValidationError("`out-surface` must not be None")
    if not isinstance(params["out-surface"], str):
        raise StyxValidationError(f'`out-surface` has the wrong type: Received `{type(params.get("out-surface", None))}` expected `str`')
    if params.get("placement-out", None) is not None:
        surface_cortex_layer_placement_out_validate(params["placement-out"])


def surface_cortex_layer_cargs(
    params: SurfaceCortexLayerParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-cortex-layer"
    ])
    cargs.append(execution.input_file(params.get("white-surface", None)))
    cargs.append(execution.input_file(params.get("pial-surface", None)))
    cargs.append(str(params.get("location", None)))
    cargs.append(params.get("out-surface", None))
    if params.get("placement-out", None) is not None:
        cargs.extend(surface_cortex_layer_placement_out_cargs(params.get("placement-out", None), execution))
    return cargs


def surface_cortex_layer_outputs(
    params: SurfaceCortexLayerParamsDict,
    execution: Execution,
) -> SurfaceCortexLayerOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCortexLayerOutputs(
        root=execution.output_file("."),
        out_surface=execution.output_file(params.get("out-surface", None)),
        placement_out=surface_cortex_layer_placement_out_outputs(params.get("placement-out"), execution) if params.get("placement-out") else None,
    )
    return ret


def surface_cortex_layer_execute(
    params: SurfaceCortexLayerParamsDict,
    runner: Runner | None = None,
) -> SurfaceCortexLayerOutputs:
    """
    CREATE SURFACE APPROXIMATING A CORTICAL LAYER.
    
    The input surfaces must have vertex correspondence. The output surface is
    generated by placing vertices between the two surfaces such that the
    enclosed volume within any small patch of the new and white surfaces is the
    given fraction of the volume of the same patch between the pial and white
    surfaces (i.e., specifying 0 would give the white surface, 1 would give the
    pial surface). .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCortexLayerOutputs`).
    """
    surface_cortex_layer_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_CORTEX_LAYER_METADATA)
    params = execution.params(params)
    cargs = surface_cortex_layer_cargs(params, execution)
    ret = surface_cortex_layer_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_cortex_layer(
    white_surface: InputPathType,
    pial_surface: InputPathType,
    location: float,
    out_surface: str,
    placement_out: SurfaceCortexLayerPlacementOutParamsDict | None = None,
    runner: Runner | None = None,
) -> SurfaceCortexLayerOutputs:
    """
    CREATE SURFACE APPROXIMATING A CORTICAL LAYER.
    
    The input surfaces must have vertex correspondence. The output surface is
    generated by placing vertices between the two surfaces such that the
    enclosed volume within any small patch of the new and white surfaces is the
    given fraction of the volume of the same patch between the pial and white
    surfaces (i.e., specifying 0 would give the white surface, 1 would give the
    pial surface). .
    
    Args:
        white_surface: the white matter surface.
        pial_surface: the pial surface.
        location: what volume fraction to place the layer at.
        out_surface: the output surface.
        placement_out: output the placement as a volume fraction from pial to\
            white.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCortexLayerOutputs`).
    """
    params = surface_cortex_layer_params(
        white_surface=white_surface,
        pial_surface=pial_surface,
        location=location,
        out_surface=out_surface,
        placement_out=placement_out,
    )
    return surface_cortex_layer_execute(params, runner)


__all__ = [
    "SURFACE_CORTEX_LAYER_METADATA",
    "SurfaceCortexLayerOutputs",
    "SurfaceCortexLayerParamsDict",
    "SurfaceCortexLayerParamsDictTagged",
    "SurfaceCortexLayerPlacementOutOutputs",
    "SurfaceCortexLayerPlacementOutParamsDict",
    "SurfaceCortexLayerPlacementOutParamsDictTagged",
    "surface_cortex_layer",
    "surface_cortex_layer_execute",
    "surface_cortex_layer_params",
    "surface_cortex_layer_placement_out",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_APPLY_WARPFIELD_METADATA = Metadata(
    id="81f972736bb6af1bd37bc10b9a2daf65cf08fc89.workbench",
    name="surface-apply-warpfield",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceApplyWarpfieldParamsDictNoTag = typing.TypedDict('_SurfaceApplyWarpfieldParamsDictNoTag', {
    "in-surf": InputPathType,
    "warpfield": str,
    "out-surf": str,
    "forward-warp": typing.NotRequired[str | None],
})
SurfaceApplyWarpfieldParamsDictTagged = typing.TypedDict('SurfaceApplyWarpfieldParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-apply-warpfield"],
    "in-surf": InputPathType,
    "warpfield": str,
    "out-surf": str,
    "forward-warp": typing.NotRequired[str | None],
})
SurfaceApplyWarpfieldParamsDict = _SurfaceApplyWarpfieldParamsDictNoTag | SurfaceApplyWarpfieldParamsDictTagged


class SurfaceApplyWarpfieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceApplyWarpfieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surf: OutputPathType
    """the output transformed surface"""


def surface_apply_warpfield_params(
    in_surf: InputPathType,
    warpfield: str,
    out_surf: str,
    forward_warp: str | None = None,
) -> SurfaceApplyWarpfieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_surf: the surface to transform.
        warpfield: the INVERSE warpfield.
        out_surf: the output transformed surface.
        forward_warp: MUST be used if using a fnirt warpfield\
            \
            the forward warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-apply-warpfield",
        "in-surf": in_surf,
        "warpfield": warpfield,
        "out-surf": out_surf,
    }
    if forward_warp is not None:
        params["forward-warp"] = forward_warp
    return params


def surface_apply_warpfield_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceApplyWarpfieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("in-surf", None) is None:
        raise StyxValidationError("`in-surf` must not be None")
    if not isinstance(params["in-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in-surf` has the wrong type: Received `{type(params.get("in-surf", None))}` expected `InputPathType`')
    if params.get("warpfield", None) is None:
        raise StyxValidationError("`warpfield` must not be None")
    if not isinstance(params["warpfield"], str):
        raise StyxValidationError(f'`warpfield` has the wrong type: Received `{type(params.get("warpfield", None))}` expected `str`')
    if params.get("out-surf", None) is None:
        raise StyxValidationError("`out-surf` must not be None")
    if not isinstance(params["out-surf"], str):
        raise StyxValidationError(f'`out-surf` has the wrong type: Received `{type(params.get("out-surf", None))}` expected `str`')
    if params.get("forward-warp", None) is not None:
        if not isinstance(params["forward-warp"], str):
            raise StyxValidationError(f'`forward-warp` has the wrong type: Received `{type(params.get("forward-warp", None))}` expected `str | None`')


def surface_apply_warpfield_cargs(
    params: SurfaceApplyWarpfieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-apply-warpfield"
    ])
    cargs.append(execution.input_file(params.get("in-surf", None)))
    cargs.append(params.get("warpfield", None))
    cargs.append(params.get("out-surf", None))
    if params.get("forward-warp", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("forward-warp", None)
        ])
    return cargs


def surface_apply_warpfield_outputs(
    params: SurfaceApplyWarpfieldParamsDict,
    execution: Execution,
) -> SurfaceApplyWarpfieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceApplyWarpfieldOutputs(
        root=execution.output_file("."),
        out_surf=execution.output_file(params.get("out-surf", None)),
    )
    return ret


def surface_apply_warpfield_execute(
    params: SurfaceApplyWarpfieldParamsDict,
    runner: Runner | None = None,
) -> SurfaceApplyWarpfieldOutputs:
    """
    APPLY WARPFIELD TO SURFACE FILE.
    
    NOTE: warping a surface requires the INVERSE of the warpfield used to warp
    the volume it lines up with. The header of the forward warp is needed by the
    -fnirt option in order to correctly interpret the displacements in the fnirt
    warpfield.
    
    If the -fnirt option is not present, the warpfield must be a nifti 'world'
    warpfield, which can be obtained with the -convert-warpfield command.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceApplyWarpfieldOutputs`).
    """
    surface_apply_warpfield_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_APPLY_WARPFIELD_METADATA)
    params = execution.params(params)
    cargs = surface_apply_warpfield_cargs(params, execution)
    ret = surface_apply_warpfield_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_apply_warpfield(
    in_surf: InputPathType,
    warpfield: str,
    out_surf: str,
    forward_warp: str | None = None,
    runner: Runner | None = None,
) -> SurfaceApplyWarpfieldOutputs:
    """
    APPLY WARPFIELD TO SURFACE FILE.
    
    NOTE: warping a surface requires the INVERSE of the warpfield used to warp
    the volume it lines up with. The header of the forward warp is needed by the
    -fnirt option in order to correctly interpret the displacements in the fnirt
    warpfield.
    
    If the -fnirt option is not present, the warpfield must be a nifti 'world'
    warpfield, which can be obtained with the -convert-warpfield command.
    
    Args:
        in_surf: the surface to transform.
        warpfield: the INVERSE warpfield.
        out_surf: the output transformed surface.
        forward_warp: MUST be used if using a fnirt warpfield\
            \
            the forward warpfield.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceApplyWarpfieldOutputs`).
    """
    params = surface_apply_warpfield_params(
        in_surf=in_surf,
        warpfield=warpfield,
        out_surf=out_surf,
        forward_warp=forward_warp,
    )
    return surface_apply_warpfield_execute(params, runner)


__all__ = [
    "SURFACE_APPLY_WARPFIELD_METADATA",
    "SurfaceApplyWarpfieldOutputs",
    "SurfaceApplyWarpfieldParamsDict",
    "SurfaceApplyWarpfieldParamsDictTagged",
    "surface_apply_warpfield",
    "surface_apply_warpfield_execute",
    "surface_apply_warpfield_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_GEODESIC_DISTANCE_ALL_TO_ALL_METADATA = Metadata(
    id="0be135c0c6479332f2fdfb0600eae50aaa990514.workbench",
    name="surface-geodesic-distance-all-to-all",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceGeodesicDistanceAllToAllParamsDictNoTag = typing.TypedDict('_SurfaceGeodesicDistanceAllToAllParamsDictNoTag', {
    "surface": InputPathType,
    "cifti-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "limit-mm": typing.NotRequired[float | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "naive": bool,
})
SurfaceGeodesicDistanceAllToAllParamsDictTagged = typing.TypedDict('SurfaceGeodesicDistanceAllToAllParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-geodesic-distance-all-to-all"],
    "surface": InputPathType,
    "cifti-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "limit-mm": typing.NotRequired[float | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "naive": bool,
})
SurfaceGeodesicDistanceAllToAllParamsDict = _SurfaceGeodesicDistanceAllToAllParamsDictNoTag | SurfaceGeodesicDistanceAllToAllParamsDictTagged


class SurfaceGeodesicDistanceAllToAllOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceGeodesicDistanceAllToAllParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """single-hemisphere dconn containing the distances"""


def surface_geodesic_distance_all_to_all_params(
    surface: InputPathType,
    cifti_out: str,
    area_metric: InputPathType | None = None,
    limit_mm: float | None = None,
    roi_metric: InputPathType | None = None,
    naive: bool = False,
) -> SurfaceGeodesicDistanceAllToAllParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute on.
        cifti_out: single-hemisphere dconn containing the distances.
        area_metric: vertex areas to use to correct the distances on a\
            group-average surface\
            \
            the corrected vertex areas, as a metric.
        limit_mm: stop at a specified distance\
            \
            distance in mm to stop at.
        roi_metric: only output distances for vertices inside an ROI\
            \
            the ROI as a metric file.
        naive: use only neighbors, don't crawl triangles (not recommended).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-geodesic-distance-all-to-all",
        "surface": surface,
        "cifti-out": cifti_out,
        "naive": naive,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    if limit_mm is not None:
        params["limit-mm"] = limit_mm
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def surface_geodesic_distance_all_to_all_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceGeodesicDistanceAllToAllParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("limit-mm", None) is not None:
        if not isinstance(params["limit-mm"], (float, int)):
            raise StyxValidationError(f'`limit-mm` has the wrong type: Received `{type(params.get("limit-mm", None))}` expected `float | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("naive", False) is None:
        raise StyxValidationError("`naive` must not be None")
    if not isinstance(params["naive"], bool):
        raise StyxValidationError(f'`naive` has the wrong type: Received `{type(params.get("naive", False))}` expected `bool`')


def surface_geodesic_distance_all_to_all_cargs(
    params: SurfaceGeodesicDistanceAllToAllParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-geodesic-distance-all-to-all"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("limit-mm", None) is not None:
        cargs.extend([
            "-limit",
            str(params.get("limit-mm", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("naive", False):
        cargs.append("-naive")
    return cargs


def surface_geodesic_distance_all_to_all_outputs(
    params: SurfaceGeodesicDistanceAllToAllParamsDict,
    execution: Execution,
) -> SurfaceGeodesicDistanceAllToAllOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceGeodesicDistanceAllToAllOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def surface_geodesic_distance_all_to_all_execute(
    params: SurfaceGeodesicDistanceAllToAllParamsDict,
    runner: Runner | None = None,
) -> SurfaceGeodesicDistanceAllToAllOutputs:
    """
    COMPUTE GEODESIC DISTANCES FROM ALL VERTICES.
    
    Computes geodesic distance from every vertex to every vertex, outputting a
    single-hemisphere dconn file. If you are only interested in a few vertices,
    see -surface-geodesic-distance. When -limit is specified, any vertex beyond
    the limit is assigned the value -1.
    
    The -roi option makes the output file smaller by not outputting distances to
    or from vertices outside the ROI, but paths are still allowed to go outside
    the ROI when finding distances to other vertices.
    
    The -corrected-areas option should be used when the input is a group average
    surface - group average surfaces have significantly less surface area than
    individual surfaces do, and therefore distances measured on them would be
    smaller than measuring them on individual surfaces. In this case, the input
    to this option should be a group average of the output of
    -surface-vertex-areas for each subject.
    
    If -naive is not specified, the algorithm uses not just immediate neighbors,
    but also neighbors derived from crawling across pairs of triangles that
    share an edge.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicDistanceAllToAllOutputs`).
    """
    surface_geodesic_distance_all_to_all_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_GEODESIC_DISTANCE_ALL_TO_ALL_METADATA)
    params = execution.params(params)
    cargs = surface_geodesic_distance_all_to_all_cargs(params, execution)
    ret = surface_geodesic_distance_all_to_all_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_geodesic_distance_all_to_all(
    surface: InputPathType,
    cifti_out: str,
    area_metric: InputPathType | None = None,
    limit_mm: float | None = None,
    roi_metric: InputPathType | None = None,
    naive: bool = False,
    runner: Runner | None = None,
) -> SurfaceGeodesicDistanceAllToAllOutputs:
    """
    COMPUTE GEODESIC DISTANCES FROM ALL VERTICES.
    
    Computes geodesic distance from every vertex to every vertex, outputting a
    single-hemisphere dconn file. If you are only interested in a few vertices,
    see -surface-geodesic-distance. When -limit is specified, any vertex beyond
    the limit is assigned the value -1.
    
    The -roi option makes the output file smaller by not outputting distances to
    or from vertices outside the ROI, but paths are still allowed to go outside
    the ROI when finding distances to other vertices.
    
    The -corrected-areas option should be used when the input is a group average
    surface - group average surfaces have significantly less surface area than
    individual surfaces do, and therefore distances measured on them would be
    smaller than measuring them on individual surfaces. In this case, the input
    to this option should be a group average of the output of
    -surface-vertex-areas for each subject.
    
    If -naive is not specified, the algorithm uses not just immediate neighbors,
    but also neighbors derived from crawling across pairs of triangles that
    share an edge.
    
    Args:
        surface: the surface to compute on.
        cifti_out: single-hemisphere dconn containing the distances.
        area_metric: vertex areas to use to correct the distances on a\
            group-average surface\
            \
            the corrected vertex areas, as a metric.
        limit_mm: stop at a specified distance\
            \
            distance in mm to stop at.
        roi_metric: only output distances for vertices inside an ROI\
            \
            the ROI as a metric file.
        naive: use only neighbors, don't crawl triangles (not recommended).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicDistanceAllToAllOutputs`).
    """
    params = surface_geodesic_distance_all_to_all_params(
        surface=surface,
        cifti_out=cifti_out,
        area_metric=area_metric,
        limit_mm=limit_mm,
        roi_metric=roi_metric,
        naive=naive,
    )
    return surface_geodesic_distance_all_to_all_execute(params, runner)


__all__ = [
    "SURFACE_GEODESIC_DISTANCE_ALL_TO_ALL_METADATA",
    "SurfaceGeodesicDistanceAllToAllOutputs",
    "SurfaceGeodesicDistanceAllToAllParamsDict",
    "SurfaceGeodesicDistanceAllToAllParamsDictTagged",
    "surface_geodesic_distance_all_to_all",
    "surface_geodesic_distance_all_to_all_execute",
    "surface_geodesic_distance_all_to_all_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_PARCEL_MAPPING_TO_LABEL_METADATA = Metadata(
    id="ce08a74db6f042fde95fefc290e8baf47feda5cc.workbench",
    name="cifti-parcel-mapping-to-label",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiParcelMappingToLabelParamsDictNoTag = typing.TypedDict('_CiftiParcelMappingToLabelParamsDictNoTag', {
    "cifti-in": InputPathType,
    "direction": str,
    "template-cifti": InputPathType,
    "dlabel-out": str,
})
CiftiParcelMappingToLabelParamsDictTagged = typing.TypedDict('CiftiParcelMappingToLabelParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-parcel-mapping-to-label"],
    "cifti-in": InputPathType,
    "direction": str,
    "template-cifti": InputPathType,
    "dlabel-out": str,
})
CiftiParcelMappingToLabelParamsDict = _CiftiParcelMappingToLabelParamsDictNoTag | CiftiParcelMappingToLabelParamsDictTagged


class CiftiParcelMappingToLabelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiParcelMappingToLabelParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    dlabel_out: OutputPathType
    """the output dense label file"""


def cifti_parcel_mapping_to_label_params(
    cifti_in: InputPathType,
    direction: str,
    template_cifti: InputPathType,
    dlabel_out: str,
) -> CiftiParcelMappingToLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input parcellated file.
        direction: which dimension to take the parcel map from, ROW or COLUMN.
        template_cifti: a cifti file with the desired dense mapping along\
            column.
        dlabel_out: the output dense label file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-parcel-mapping-to-label",
        "cifti-in": cifti_in,
        "direction": direction,
        "template-cifti": template_cifti,
        "dlabel-out": dlabel_out,
    }
    return params


def cifti_parcel_mapping_to_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiParcelMappingToLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("template-cifti", None) is None:
        raise StyxValidationError("`template-cifti` must not be None")
    if not isinstance(params["template-cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template-cifti` has the wrong type: Received `{type(params.get("template-cifti", None))}` expected `InputPathType`')
    if params.get("dlabel-out", None) is None:
        raise StyxValidationError("`dlabel-out` must not be None")
    if not isinstance(params["dlabel-out"], str):
        raise StyxValidationError(f'`dlabel-out` has the wrong type: Received `{type(params.get("dlabel-out", None))}` expected `str`')


def cifti_parcel_mapping_to_label_cargs(
    params: CiftiParcelMappingToLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-parcel-mapping-to-label"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(execution.input_file(params.get("template-cifti", None)))
    cargs.append(params.get("dlabel-out", None))
    return cargs


def cifti_parcel_mapping_to_label_outputs(
    params: CiftiParcelMappingToLabelParamsDict,
    execution: Execution,
) -> CiftiParcelMappingToLabelOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiParcelMappingToLabelOutputs(
        root=execution.output_file("."),
        dlabel_out=execution.output_file(params.get("dlabel-out", None)),
    )
    return ret


def cifti_parcel_mapping_to_label_execute(
    params: CiftiParcelMappingToLabelParamsDict,
    runner: Runner | None = None,
) -> CiftiParcelMappingToLabelOutputs:
    """
    CREATE DLABEL FROM PARCELLATED FILE.
    
    This command will output a dlabel file, useful for doing the same
    parcellation to another dense file.
    
    For ptseries, pscalar, plabel, pconn, and pdconn, using COLUMN for
    <direction> will work.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcelMappingToLabelOutputs`).
    """
    cifti_parcel_mapping_to_label_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_PARCEL_MAPPING_TO_LABEL_METADATA)
    params = execution.params(params)
    cargs = cifti_parcel_mapping_to_label_cargs(params, execution)
    ret = cifti_parcel_mapping_to_label_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_parcel_mapping_to_label(
    cifti_in: InputPathType,
    direction: str,
    template_cifti: InputPathType,
    dlabel_out: str,
    runner: Runner | None = None,
) -> CiftiParcelMappingToLabelOutputs:
    """
    CREATE DLABEL FROM PARCELLATED FILE.
    
    This command will output a dlabel file, useful for doing the same
    parcellation to another dense file.
    
    For ptseries, pscalar, plabel, pconn, and pdconn, using COLUMN for
    <direction> will work.
    
    Args:
        cifti_in: the input parcellated file.
        direction: which dimension to take the parcel map from, ROW or COLUMN.
        template_cifti: a cifti file with the desired dense mapping along\
            column.
        dlabel_out: the output dense label file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcelMappingToLabelOutputs`).
    """
    params = cifti_parcel_mapping_to_label_params(
        cifti_in=cifti_in,
        direction=direction,
        template_cifti=template_cifti,
        dlabel_out=dlabel_out,
    )
    return cifti_parcel_mapping_to_label_execute(params, runner)


__all__ = [
    "CIFTI_PARCEL_MAPPING_TO_LABEL_METADATA",
    "CiftiParcelMappingToLabelOutputs",
    "CiftiParcelMappingToLabelParamsDict",
    "CiftiParcelMappingToLabelParamsDictTagged",
    "cifti_parcel_mapping_to_label",
    "cifti_parcel_mapping_to_label_execute",
    "cifti_parcel_mapping_to_label_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BORDER_RESAMPLE_METADATA = Metadata(
    id="323bcae88f347c08d0669f3360e5db16c8affb3e.workbench",
    name="border-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BorderResampleParamsDictNoTag = typing.TypedDict('_BorderResampleParamsDictNoTag', {
    "border-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "border-out": str,
})
BorderResampleParamsDictTagged = typing.TypedDict('BorderResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/border-resample"],
    "border-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "border-out": str,
})
BorderResampleParamsDict = _BorderResampleParamsDictNoTag | BorderResampleParamsDictTagged


class BorderResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BorderResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    border_out: OutputPathType
    """the output border file"""


def border_resample_params(
    border_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    border_out: str,
) -> BorderResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border_in: the border file to resample.
        current_sphere: a sphere surface with the mesh that the metric is\
            currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        border_out: the output border file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/border-resample",
        "border-in": border_in,
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
        "border-out": border_out,
    }
    return params


def border_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border-in", None) is None:
        raise StyxValidationError("`border-in` must not be None")
    if not isinstance(params["border-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border-in` has the wrong type: Received `{type(params.get("border-in", None))}` expected `InputPathType`')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("border-out", None) is None:
        raise StyxValidationError("`border-out` must not be None")
    if not isinstance(params["border-out"], str):
        raise StyxValidationError(f'`border-out` has the wrong type: Received `{type(params.get("border-out", None))}` expected `str`')


def border_resample_cargs(
    params: BorderResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-border-resample"
    ])
    cargs.append(execution.input_file(params.get("border-in", None)))
    cargs.append(execution.input_file(params.get("current-sphere", None)))
    cargs.append(execution.input_file(params.get("new-sphere", None)))
    cargs.append(params.get("border-out", None))
    return cargs


def border_resample_outputs(
    params: BorderResampleParamsDict,
    execution: Execution,
) -> BorderResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BorderResampleOutputs(
        root=execution.output_file("."),
        border_out=execution.output_file(params.get("border-out", None)),
    )
    return ret


def border_resample_execute(
    params: BorderResampleParamsDict,
    runner: Runner | None = None,
) -> BorderResampleOutputs:
    """
    RESAMPLE A BORDER FILE TO A DIFFERENT MESH.
    
    Resamples a border file, given two spherical surfaces that are in register.
    Only borders that have the same structure as current-sphere will be
    resampled.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderResampleOutputs`).
    """
    border_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BORDER_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = border_resample_cargs(params, execution)
    ret = border_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def border_resample(
    border_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    border_out: str,
    runner: Runner | None = None,
) -> BorderResampleOutputs:
    """
    RESAMPLE A BORDER FILE TO A DIFFERENT MESH.
    
    Resamples a border file, given two spherical surfaces that are in register.
    Only borders that have the same structure as current-sphere will be
    resampled.
    
    Args:
        border_in: the border file to resample.
        current_sphere: a sphere surface with the mesh that the metric is\
            currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        border_out: the output border file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderResampleOutputs`).
    """
    params = border_resample_params(
        border_in=border_in,
        current_sphere=current_sphere,
        new_sphere=new_sphere,
        border_out=border_out,
    )
    return border_resample_execute(params, runner)


__all__ = [
    "BORDER_RESAMPLE_METADATA",
    "BorderResampleOutputs",
    "BorderResampleParamsDict",
    "BorderResampleParamsDictTagged",
    "border_resample",
    "border_resample_execute",
    "border_resample_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_AVERAGE_METADATA = Metadata(
    id="3cbdda68e7c781752cdffcc794c5ac24ffea464a.workbench",
    name="surface-average",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceAverageStddevParamsDictNoTag = typing.TypedDict('_SurfaceAverageStddevParamsDictNoTag', {
    "stddev-metric-out": str,
})
SurfaceAverageStddevParamsDictTagged = typing.TypedDict('SurfaceAverageStddevParamsDictTagged', {
    "@type": typing.Literal["stddev"],
    "stddev-metric-out": str,
})
SurfaceAverageStddevParamsDict = _SurfaceAverageStddevParamsDictNoTag | SurfaceAverageStddevParamsDictTagged


_SurfaceAverageUncertaintyParamsDictNoTag = typing.TypedDict('_SurfaceAverageUncertaintyParamsDictNoTag', {
    "uncert-metric-out": str,
})
SurfaceAverageUncertaintyParamsDictTagged = typing.TypedDict('SurfaceAverageUncertaintyParamsDictTagged', {
    "@type": typing.Literal["uncertainty"],
    "uncert-metric-out": str,
})
SurfaceAverageUncertaintyParamsDict = _SurfaceAverageUncertaintyParamsDictNoTag | SurfaceAverageUncertaintyParamsDictTagged


_SurfaceAverageSurfParamsDictNoTag = typing.TypedDict('_SurfaceAverageSurfParamsDictNoTag', {
    "surface": InputPathType,
    "weight": typing.NotRequired[float | None],
})
SurfaceAverageSurfParamsDictTagged = typing.TypedDict('SurfaceAverageSurfParamsDictTagged', {
    "@type": typing.Literal["surf"],
    "surface": InputPathType,
    "weight": typing.NotRequired[float | None],
})
SurfaceAverageSurfParamsDict = _SurfaceAverageSurfParamsDictNoTag | SurfaceAverageSurfParamsDictTagged


_SurfaceAverageParamsDictNoTag = typing.TypedDict('_SurfaceAverageParamsDictNoTag', {
    "surface-out": str,
    "stddev": typing.NotRequired[SurfaceAverageStddevParamsDict | None],
    "uncertainty": typing.NotRequired[SurfaceAverageUncertaintyParamsDict | None],
    "surf": typing.NotRequired[list[SurfaceAverageSurfParamsDict] | None],
})
SurfaceAverageParamsDictTagged = typing.TypedDict('SurfaceAverageParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-average"],
    "surface-out": str,
    "stddev": typing.NotRequired[SurfaceAverageStddevParamsDict | None],
    "uncertainty": typing.NotRequired[SurfaceAverageUncertaintyParamsDict | None],
    "surf": typing.NotRequired[list[SurfaceAverageSurfParamsDict] | None],
})
SurfaceAverageParamsDict = _SurfaceAverageParamsDictNoTag | SurfaceAverageParamsDictTagged


class SurfaceAverageStddevOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceAverageStddevParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    stddev_metric_out: OutputPathType
    """the output metric for 3D sample standard deviation"""


def surface_average_stddev(
    stddev_metric_out: str,
) -> SurfaceAverageStddevParamsDictTagged:
    """
    Build parameters.
    
    Args:
        stddev_metric_out: the output metric for 3D sample standard deviation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "stddev",
        "stddev-metric-out": stddev_metric_out,
    }
    return params


def surface_average_stddev_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceAverageStddevParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("stddev-metric-out", None) is None:
        raise StyxValidationError("`stddev-metric-out` must not be None")
    if not isinstance(params["stddev-metric-out"], str):
        raise StyxValidationError(f'`stddev-metric-out` has the wrong type: Received `{type(params.get("stddev-metric-out", None))}` expected `str`')


def surface_average_stddev_cargs(
    params: SurfaceAverageStddevParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-stddev",
        params.get("stddev-metric-out", None)
    ])
    return cargs


def surface_average_stddev_outputs(
    params: SurfaceAverageStddevParamsDict,
    execution: Execution,
) -> SurfaceAverageStddevOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceAverageStddevOutputs(
        root=execution.output_file("."),
        stddev_metric_out=execution.output_file(params.get("stddev-metric-out", None)),
    )
    return ret


class SurfaceAverageUncertaintyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceAverageUncertaintyParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    uncert_metric_out: OutputPathType
    """the output metric for uncertainty"""


def surface_average_uncertainty(
    uncert_metric_out: str,
) -> SurfaceAverageUncertaintyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        uncert_metric_out: the output metric for uncertainty.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "uncertainty",
        "uncert-metric-out": uncert_metric_out,
    }
    return params


def surface_average_uncertainty_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceAverageUncertaintyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("uncert-metric-out", None) is None:
        raise StyxValidationError("`uncert-metric-out` must not be None")
    if not isinstance(params["uncert-metric-out"], str):
        raise StyxValidationError(f'`uncert-metric-out` has the wrong type: Received `{type(params.get("uncert-metric-out", None))}` expected `str`')


def surface_average_uncertainty_cargs(
    params: SurfaceAverageUncertaintyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-uncertainty",
        params.get("uncert-metric-out", None)
    ])
    return cargs


def surface_average_uncertainty_outputs(
    params: SurfaceAverageUncertaintyParamsDict,
    execution: Execution,
) -> SurfaceAverageUncertaintyOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceAverageUncertaintyOutputs(
        root=execution.output_file("."),
        uncert_metric_out=execution.output_file(params.get("uncert-metric-out", None)),
    )
    return ret


def surface_average_surf(
    surface: InputPathType,
    weight: float | None = None,
) -> SurfaceAverageSurfParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: a surface file to average.
        weight: specify a weighted average\
            \
            the weight to use (default 1).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surf",
        "surface": surface,
    }
    if weight is not None:
        params["weight"] = weight
    return params


def surface_average_surf_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceAverageSurfParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("weight", None) is not None:
        if not isinstance(params["weight"], (float, int)):
            raise StyxValidationError(f'`weight` has the wrong type: Received `{type(params.get("weight", None))}` expected `float | None`')


def surface_average_surf_cargs(
    params: SurfaceAverageSurfParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surf",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("weight", None) is not None:
        cargs.extend([
            "-weight",
            str(params.get("weight", None))
        ])
    return cargs


class SurfaceAverageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceAverageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the output averaged surface"""
    stddev: SurfaceAverageStddevOutputs | None
    """Outputs from `surface_average_stddev_outputs`."""
    uncertainty: SurfaceAverageUncertaintyOutputs | None
    """Outputs from `surface_average_uncertainty_outputs`."""


def surface_average_params(
    surface_out: str,
    stddev: SurfaceAverageStddevParamsDict | None = None,
    uncertainty: SurfaceAverageUncertaintyParamsDict | None = None,
    surf: list[SurfaceAverageSurfParamsDict] | None = None,
) -> SurfaceAverageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_out: the output averaged surface.
        stddev: compute 3D sample standard deviation.
        uncertainty: compute caret5 'uncertainty'.
        surf: specify a surface to include in the average.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-average",
        "surface-out": surface_out,
    }
    if stddev is not None:
        params["stddev"] = stddev
    if uncertainty is not None:
        params["uncertainty"] = uncertainty
    if surf is not None:
        params["surf"] = surf
    return params


def surface_average_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceAverageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')
    if params.get("stddev", None) is not None:
        surface_average_stddev_validate(params["stddev"])
    if params.get("uncertainty", None) is not None:
        surface_average_uncertainty_validate(params["uncertainty"])
    if params.get("surf", None) is not None:
        if not isinstance(params["surf"], list):
            raise StyxValidationError(f'`surf` has the wrong type: Received `{type(params.get("surf", None))}` expected `list[SurfaceAverageSurfParamsDict] | None`')
        for e in params["surf"]:
            surface_average_surf_validate(e)


def surface_average_cargs(
    params: SurfaceAverageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-average"
    ])
    cargs.append(params.get("surface-out", None))
    if params.get("stddev", None) is not None or params.get("uncertainty", None) is not None or params.get("surf", None) is not None:
        cargs.extend([
            *(surface_average_stddev_cargs(params.get("stddev", None), execution) if (params.get("stddev", None) is not None) else []),
            *(surface_average_uncertainty_cargs(params.get("uncertainty", None), execution) if (params.get("uncertainty", None) is not None) else []),
            *([a for c in [surface_average_surf_cargs(s, execution) for s in params.get("surf", None)] for a in c] if (params.get("surf", None) is not None) else [])
        ])
    return cargs


def surface_average_outputs(
    params: SurfaceAverageParamsDict,
    execution: Execution,
) -> SurfaceAverageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceAverageOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
        stddev=surface_average_stddev_outputs(params.get("stddev"), execution) if params.get("stddev") else None,
        uncertainty=surface_average_uncertainty_outputs(params.get("uncertainty"), execution) if params.get("uncertainty") else None,
    )
    return ret


def surface_average_execute(
    params: SurfaceAverageParamsDict,
    runner: Runner | None = None,
) -> SurfaceAverageOutputs:
    """
    AVERAGE SURFACE FILES TOGETHER.
    
    The 3D sample standard deviation is computed as 'sqrt(sum(squaredlength(xyz
    - mean(xyz)))/(n - 1))'.
    
    Uncertainty is a legacy measure used in caret5, and is computed as
    'sum(length(xyz - mean(xyz)))/n'.
    
    When weights are used, the 3D sample standard deviation treats them as
    reliability weights.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceAverageOutputs`).
    """
    surface_average_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_AVERAGE_METADATA)
    params = execution.params(params)
    cargs = surface_average_cargs(params, execution)
    ret = surface_average_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_average(
    surface_out: str,
    stddev: SurfaceAverageStddevParamsDict | None = None,
    uncertainty: SurfaceAverageUncertaintyParamsDict | None = None,
    surf: list[SurfaceAverageSurfParamsDict] | None = None,
    runner: Runner | None = None,
) -> SurfaceAverageOutputs:
    """
    AVERAGE SURFACE FILES TOGETHER.
    
    The 3D sample standard deviation is computed as 'sqrt(sum(squaredlength(xyz
    - mean(xyz)))/(n - 1))'.
    
    Uncertainty is a legacy measure used in caret5, and is computed as
    'sum(length(xyz - mean(xyz)))/n'.
    
    When weights are used, the 3D sample standard deviation treats them as
    reliability weights.
    
    Args:
        surface_out: the output averaged surface.
        stddev: compute 3D sample standard deviation.
        uncertainty: compute caret5 'uncertainty'.
        surf: specify a surface to include in the average.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceAverageOutputs`).
    """
    params = surface_average_params(
        surface_out=surface_out,
        stddev=stddev,
        uncertainty=uncertainty,
        surf=surf,
    )
    return surface_average_execute(params, runner)


__all__ = [
    "SURFACE_AVERAGE_METADATA",
    "SurfaceAverageOutputs",
    "SurfaceAverageParamsDict",
    "SurfaceAverageParamsDictTagged",
    "SurfaceAverageStddevOutputs",
    "SurfaceAverageStddevParamsDict",
    "SurfaceAverageStddevParamsDictTagged",
    "SurfaceAverageSurfParamsDict",
    "SurfaceAverageSurfParamsDictTagged",
    "SurfaceAverageUncertaintyOutputs",
    "SurfaceAverageUncertaintyParamsDict",
    "SurfaceAverageUncertaintyParamsDictTagged",
    "surface_average",
    "surface_average_execute",
    "surface_average_params",
    "surface_average_stddev",
    "surface_average_surf",
    "surface_average_uncertainty",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METADATA_REMOVE_PROVENANCE_METADATA = Metadata(
    id="e8c16ad9420831ac3f90677352afe3ddc5dcbac7.workbench",
    name="metadata-remove-provenance",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetadataRemoveProvenanceParamsDictNoTag = typing.TypedDict('_MetadataRemoveProvenanceParamsDictNoTag', {
    "input-file": str,
    "output-file": str,
})
MetadataRemoveProvenanceParamsDictTagged = typing.TypedDict('MetadataRemoveProvenanceParamsDictTagged', {
    "@type": typing.Literal["workbench/metadata-remove-provenance"],
    "input-file": str,
    "output-file": str,
})
MetadataRemoveProvenanceParamsDict = _MetadataRemoveProvenanceParamsDictNoTag | MetadataRemoveProvenanceParamsDictTagged


class MetadataRemoveProvenanceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetadataRemoveProvenanceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metadata_remove_provenance_params(
    input_file: str,
    output_file: str,
) -> MetadataRemoveProvenanceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_file: the file to remove provenance information from.
        output_file: output - the name to save the modified file as.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metadata-remove-provenance",
        "input-file": input_file,
        "output-file": output_file,
    }
    return params


def metadata_remove_provenance_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetadataRemoveProvenanceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input-file", None) is None:
        raise StyxValidationError("`input-file` must not be None")
    if not isinstance(params["input-file"], str):
        raise StyxValidationError(f'`input-file` has the wrong type: Received `{type(params.get("input-file", None))}` expected `str`')
    if params.get("output-file", None) is None:
        raise StyxValidationError("`output-file` must not be None")
    if not isinstance(params["output-file"], str):
        raise StyxValidationError(f'`output-file` has the wrong type: Received `{type(params.get("output-file", None))}` expected `str`')


def metadata_remove_provenance_cargs(
    params: MetadataRemoveProvenanceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metadata-remove-provenance"
    ])
    cargs.append(params.get("input-file", None))
    cargs.append(params.get("output-file", None))
    return cargs


def metadata_remove_provenance_outputs(
    params: MetadataRemoveProvenanceParamsDict,
    execution: Execution,
) -> MetadataRemoveProvenanceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetadataRemoveProvenanceOutputs(
        root=execution.output_file("."),
    )
    return ret


def metadata_remove_provenance_execute(
    params: MetadataRemoveProvenanceParamsDict,
    runner: Runner | None = None,
) -> MetadataRemoveProvenanceOutputs:
    """
    REMOVE PROVENANCE INFORMATION FROM FILE METADATA.
    
    Removes the provenance metadata fields added by workbench during processing.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetadataRemoveProvenanceOutputs`).
    """
    metadata_remove_provenance_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METADATA_REMOVE_PROVENANCE_METADATA)
    params = execution.params(params)
    cargs = metadata_remove_provenance_cargs(params, execution)
    ret = metadata_remove_provenance_outputs(params, execution)
    execution.run(cargs)
    return ret


def metadata_remove_provenance(
    input_file: str,
    output_file: str,
    runner: Runner | None = None,
) -> MetadataRemoveProvenanceOutputs:
    """
    REMOVE PROVENANCE INFORMATION FROM FILE METADATA.
    
    Removes the provenance metadata fields added by workbench during processing.
    
    Args:
        input_file: the file to remove provenance information from.
        output_file: output - the name to save the modified file as.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetadataRemoveProvenanceOutputs`).
    """
    params = metadata_remove_provenance_params(
        input_file=input_file,
        output_file=output_file,
    )
    return metadata_remove_provenance_execute(params, runner)


__all__ = [
    "METADATA_REMOVE_PROVENANCE_METADATA",
    "MetadataRemoveProvenanceOutputs",
    "MetadataRemoveProvenanceParamsDict",
    "MetadataRemoveProvenanceParamsDictTagged",
    "metadata_remove_provenance",
    "metadata_remove_provenance_execute",
    "metadata_remove_provenance_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_PARCEL_SMOOTHING_METADATA = Metadata(
    id="4531678f8f3118c9ec57f1efd95f5ab4ee4503d2.workbench",
    name="volume-parcel-smoothing",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeParcelSmoothingParamsDictNoTag = typing.TypedDict('_VolumeParcelSmoothingParamsDictNoTag', {
    "data-volume": InputPathType,
    "label-volume": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "fix-zeros": bool,
    "fwhm": bool,
})
VolumeParcelSmoothingParamsDictTagged = typing.TypedDict('VolumeParcelSmoothingParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-parcel-smoothing"],
    "data-volume": InputPathType,
    "label-volume": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "fix-zeros": bool,
    "fwhm": bool,
})
VolumeParcelSmoothingParamsDict = _VolumeParcelSmoothingParamsDictNoTag | VolumeParcelSmoothingParamsDictTagged


class VolumeParcelSmoothingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeParcelSmoothingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_parcel_smoothing_params(
    data_volume: InputPathType,
    label_volume: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    fix_zeros: bool = False,
    fwhm: bool = False,
) -> VolumeParcelSmoothingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_volume: the volume to smooth.
        label_volume: a label volume containing the parcels to smooth.
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        volume_out: the output volume.
        subvol: select a single subvolume to smooth\
            \
            the subvolume number or name.
        fix_zeros: treat zero values as not being data.
        fwhm: smoothing kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-parcel-smoothing",
        "data-volume": data_volume,
        "label-volume": label_volume,
        "kernel": kernel,
        "volume-out": volume_out,
        "fix-zeros": fix_zeros,
        "fwhm": fwhm,
    }
    if subvol is not None:
        params["subvol"] = subvol
    return params


def volume_parcel_smoothing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeParcelSmoothingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-volume", None) is None:
        raise StyxValidationError("`data-volume` must not be None")
    if not isinstance(params["data-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`data-volume` has the wrong type: Received `{type(params.get("data-volume", None))}` expected `InputPathType`')
    if params.get("label-volume", None) is None:
        raise StyxValidationError("`label-volume` must not be None")
    if not isinstance(params["label-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-volume` has the wrong type: Received `{type(params.get("label-volume", None))}` expected `InputPathType`')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("fix-zeros", False) is None:
        raise StyxValidationError("`fix-zeros` must not be None")
    if not isinstance(params["fix-zeros"], bool):
        raise StyxValidationError(f'`fix-zeros` has the wrong type: Received `{type(params.get("fix-zeros", False))}` expected `bool`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_parcel_smoothing_cargs(
    params: VolumeParcelSmoothingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-parcel-smoothing"
    ])
    cargs.append(execution.input_file(params.get("data-volume", None)))
    cargs.append(execution.input_file(params.get("label-volume", None)))
    cargs.append(str(params.get("kernel", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("fix-zeros", False):
        cargs.append("-fix-zeros")
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def volume_parcel_smoothing_outputs(
    params: VolumeParcelSmoothingParamsDict,
    execution: Execution,
) -> VolumeParcelSmoothingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeParcelSmoothingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_parcel_smoothing_execute(
    params: VolumeParcelSmoothingParamsDict,
    runner: Runner | None = None,
) -> VolumeParcelSmoothingOutputs:
    """
    SMOOTH PARCELS IN A VOLUME SEPARATELY.
    
    The volume is smoothed within each label in the label volume using data only
    from within the label. Equivalent to running volume smoothing with ROIs
    matching each label separately, then adding the resulting volumes, but
    faster.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelSmoothingOutputs`).
    """
    volume_parcel_smoothing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_PARCEL_SMOOTHING_METADATA)
    params = execution.params(params)
    cargs = volume_parcel_smoothing_cargs(params, execution)
    ret = volume_parcel_smoothing_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_parcel_smoothing(
    data_volume: InputPathType,
    label_volume: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    fix_zeros: bool = False,
    fwhm: bool = False,
    runner: Runner | None = None,
) -> VolumeParcelSmoothingOutputs:
    """
    SMOOTH PARCELS IN A VOLUME SEPARATELY.
    
    The volume is smoothed within each label in the label volume using data only
    from within the label. Equivalent to running volume smoothing with ROIs
    matching each label separately, then adding the resulting volumes, but
    faster.
    
    Args:
        data_volume: the volume to smooth.
        label_volume: a label volume containing the parcels to smooth.
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        volume_out: the output volume.
        subvol: select a single subvolume to smooth\
            \
            the subvolume number or name.
        fix_zeros: treat zero values as not being data.
        fwhm: smoothing kernel size is FWHM, not sigma.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelSmoothingOutputs`).
    """
    params = volume_parcel_smoothing_params(
        data_volume=data_volume,
        label_volume=label_volume,
        kernel=kernel,
        volume_out=volume_out,
        subvol=subvol,
        fix_zeros=fix_zeros,
        fwhm=fwhm,
    )
    return volume_parcel_smoothing_execute(params, runner)


__all__ = [
    "VOLUME_PARCEL_SMOOTHING_METADATA",
    "VolumeParcelSmoothingOutputs",
    "VolumeParcelSmoothingParamsDict",
    "VolumeParcelSmoothingParamsDictTagged",
    "volume_parcel_smoothing",
    "volume_parcel_smoothing_execute",
    "volume_parcel_smoothing_params",
]
